//
//  PaymentViewModel.swift
//  RoomscannerApp
//
//  Created by Asil Arslan on 28.08.2023.
//

import Foundation
import StoreKit
import SwiftyStoreKit

class PaymentViewModel: ObservableObject{
    
    @Published var isLoadingRetrieveProducts = false
    @Published var isLoadingPayment = false
    @Published var showAlert = false
    @Published var title: String = ""
    @Published var payments = [Payment]()
    @Published var selectedProductId: String = ""
    @Published var selectedPayment: Payment?
    
    init(){
        initConfiguration()
        let monthly = Payment(paymentId: PaymentId.monthly, title: "", index: 0, price: "", discount: "", extraInfo: "", unit: "1", duration: "Monthly")
        let yearly = Payment(paymentId: PaymentId.yearly, title: "", index: 1, price: "", discount: "", extraInfo: "", unit: "1", duration: "Yearly")
        payments.append(monthly)
        payments.append(yearly)
        selectedPayment = yearly
        selectedProductId = yearly.paymentId
    }
    
    func initConfiguration(){
//        if AppUserDefaults.isPremiumUser {
//            checkSubscription(productId: AppUserDefaults.paymentId)
//        }
        
        retrieveProducts()
    }
    
    func checkSubscription(productId: String){
        AppUserDefaults.isPremiumUser = true      
    }
    
    func checkRestoreSubscription(productId: String){
        let appleValidator = AppleReceiptValidator(service: .production, sharedSecret: "0bfbb73ccc354711af0e295d101864a2")
        SwiftyStoreKit.verifyReceipt(using: appleValidator) { result in
            switch result {
            case .success(let receipt):
                // Verify the purchase of a Subscription
                let purchaseResult = SwiftyStoreKit.verifySubscription(
                    ofType: .autoRenewable, // or .nonRenewing (see below)
                    productId: productId,
                    inReceipt: receipt)
                    
                switch purchaseResult {
                case .purchased(let expiryDate, let items):
                    print("\(productId) is valid until \(expiryDate)\n\(items)\n")
                    self.showAlert(title: "Restore success")
                    AppUserDefaults.isPremiumUser = true
                case .expired(let expiryDate, let items):
                    print("\(productId) is expired since \(expiryDate)\n\(items)\n")
                    self.showAlert(title: "Subscription is expired")
                    AppUserDefaults.isPremiumUser = false
                case .notPurchased:
                    print("The user has never purchased \(productId)")
                    self.showAlert(title: "Not purchased")
                    AppUserDefaults.isPremiumUser = false
                }

            case .error(let error):
                print("Receipt verification failed: \(error)")
                AppUserDefaults.isPremiumUser = false
            }
        }
    }
    
    func retrieveProducts(){
        isLoadingRetrieveProducts = true
        SwiftyStoreKit.retrieveProductsInfo([PaymentId.monthly,PaymentId.yearly]) { result in
            self.isLoadingRetrieveProducts = false
            result.retrievedProducts.forEach { skProduct in
                
                switch(skProduct.productIdentifier){
                case PaymentId.monthly:
                    self.payments[0].price = skProduct.price.doubleValue
                    self.payments[0].priceLocale = skProduct.localizedPrice ?? "\(skProduct.price)"
                    self.payments[0].currency = skProduct.priceLocale.currencySymbol ?? "USD"
                    self.payments[0].isTrialAvailable = skProduct.introductoryPrice != nil
                    self.payments[0].trialPrice =  skProduct.introductoryPrice() ?? "Not Available"
                    self.payments[0].locale =  skProduct.priceLocale
                    break;
                case PaymentId.yearly:
                    self.payments[1].price = skProduct.price.doubleValue
                    self.payments[1].priceLocale = skProduct.localizedPrice ?? "\(skProduct.price)"
                    self.payments[1].currency = skProduct.priceLocale.currencySymbol ?? "USD"
                    let monthlyPrice = String(format: "%.1f",Float(skProduct.price) / 12)
                    let currency = skProduct.priceLocale.currencySymbol ?? ""
                    self.payments[1].discount = "\(currency)\(monthlyPrice) / \("_mo".localized) "
                    self.payments[1].isTrialAvailable = skProduct.introductoryPrice != nil
                    self.payments[1].trialPrice =  skProduct.introductoryPrice() ?? "Not Available"
                    self.payments[1].locale =  skProduct.priceLocale
                    break;
                default:
                    debugPrint("PaymentViewModel default")
                    break;
                }
            }
        }
        
        
        selectedProductId = AppConfig.YEARLY_PRODUCT_ID
        if let product = payments.filter( {$0.paymentId == selectedProductId}).first {
            selectedPayment = product
        }
        
    }
    
    func startPayment(with paymentId: String, completion: @escaping (PaymentResult) -> Void){
        isLoadingPayment = true
        SwiftyStoreKit.purchaseProduct(paymentId, quantity: 1, atomically: true) { result in
            debugPrint("PaymentManager startPayment => \(paymentId)")
            switch result {
            case .success(let purchase):
                debugPrint("PaymentManager success => \(purchase.productId)")
                completion(PaymentResult(success: true,error: nil))
            case .error(let error):
                if let _error = PaymentErrorHandler.map(from: error){
                    debugPrint("PaymentManager error => \(_error)")
                    completion(PaymentResult(success: false,error: _error))
                }else {
                    completion(PaymentResult(success: false,error: nil))
                }
            case .deferred(purchase: let purchase):
                debugPrint("PaymentManager deferred => \(purchase.productId)")
                completion(PaymentResult(success: false,error: nil))
            }
            self.isLoadingPayment = false
            
        }
    }
    
    private func showAlert(title: String){
        self.title = title
        self.showAlert = true
    }
    
    func restorePurchases(completion: @escaping (PaymentResult) -> Void){
        debugPrint("PaymentManager restorePurchases")
        isLoadingPayment = true
        
        SwiftyStoreKit.restorePurchases(atomically: true) { results in
            debugPrint("PaymentManager restorePurchases results => \(results)")
            if results.restoreFailedPurchases.count > 0 {
                self.showAlert(title: "No restored purchase")
                completion(PaymentResult(success: false,error: nil))
            }
            else if results.restoredPurchases.count > 0 {
                if let purchase =  results.restoredPurchases.first{
                    self.checkRestoreSubscription(productId: purchase.productId)
//                    self.startPayment(with: purchase.productId, completion: completion)
                }
            }
            else {
                self.showAlert(title: "No restored purchase")
                completion(PaymentResult(success: false,error: nil))
            }
            self.isLoadingPayment = false
        }
    }
    
    func saveValue() -> Double{
        var monthly: Double = 0
        var yearly: Double = 0
        if let product = payments.filter( {$0.paymentId == PaymentId.monthly}).first {
            monthly = product.price
        }
        if let product = payments.filter( {$0.paymentId == PaymentId.yearly}).first {
            yearly = product.price
        }
        return 100 * (monthly - yearly / 12) / monthly
    }
    
}

struct PaymentResult{
    var success: Bool
    var error:String?
}



struct PaymentErrorHandler{
    static func map(from error: SKError) -> String? {
        var errorDescription : String? = nil
        switch error.code {
        case .unknown:
            errorDescription = "Unknown error. Please contact support"
        case .clientInvalid:
            errorDescription = "Not allowed to make the payment"
        case .paymentCancelled:
            errorDescription = "Payment cancelled"
        case .paymentInvalid:
            errorDescription = "The purchase identifier was invalid"
        case .paymentNotAllowed:
            errorDescription = "The device is not allowed to make the payment"
        case .storeProductNotAvailable:
            errorDescription = "The product is not available in the current storefront"
        case .cloudServicePermissionDenied:
            errorDescription = "Access to cloud service information is not allowed"
        case .cloudServiceNetworkConnectionFailed:
            errorDescription = "Could not connect to the network"
        case .cloudServiceRevoked:
            errorDescription = "User has revoked permission to use this cloud service"
        default:
            errorDescription = (error as NSError).localizedDescription
        }
        return errorDescription
    }
}


extension String{
    var localized: String {
        return NSLocalizedString(self, comment: "\(self)_comment")
    }
}


class PaymentId{
    static let shared = PaymentId()
    
    static let monthly = AppConfig.MONTHLY_PRODUCT_ID
    static let yearly = AppConfig.YEARLY_PRODUCT_ID
    
    private init() {}
}
extension SKProduct {
    
        func introductoryPrice() -> String?{
            if introductoryPrice != nil {
    //            let formatter = NumberFormatter()
    //            formatter.numberStyle = .currency
    //            formatter.locale = introductoryPrice!.priceLocale
    //            let text = formatter.string(from: price)
                let period = introductoryPrice!.subscriptionPeriod.unit
                var periodString = ""
                switch period {
                case .day:
                    periodString = "day"
                case .month:
                    periodString = "month"
                case .week:
                    periodString = "week"
                case .year:
                    periodString = "year"
                default:
                    break
                }
                let unitCount = introductoryPrice!.subscriptionPeriod.numberOfUnits
                let unitString = unitCount == 1 ? periodString : "\(unitCount) \(periodString)s"
                return "\(unitString) for " + "FREE"//(text ?? "")
            }
            return nil
        }
}

