//
//  PaywallView.swift
//  WordPress
//
//  Created by Studio on 1.01.2023.
//

import SwiftUI

struct PaywallView: View {
    @EnvironmentObject var paymentViewModel: PaymentViewModel
    @EnvironmentObject var settingsViewModel: SettingsViewModel
    @Environment(\.dismiss) private var dismiss
    @State private var selectedPlan: PremiumPlan = .yearly
    @State private var showAlert = false
    @State private var alertText = ""
    //    let iPhone14ProMaxScreenSize: CGSize = CGSize(width: 430.0, height: 932.0)
    let screenSize: CGSize = UIScreen.main.bounds.size
    var features: [String] = AppConfig.FEATURES
    var monthsProductId: String = AppConfig.MONTHLY_PRODUCT_ID
    var oneYearProductId: String = AppConfig.YEARLY_PRODUCT_ID
    
    private let privacyPolicyURL: URL = URL(string: AppConfig.PRIVACY_POLICIY_URL)!
    private let termsAndConditionsURL: URL = URL(string: AppConfig.TERMS_AND_CONDITIONS_URL)!
    
    
    
    var body: some View {
        ZStack {
            Color(uiColor: .systemGroupedBackground).ignoresSafeArea()
            ScrollView(showsIndicators: false){
                VStack(alignment: .center, spacing: 16) {
                    headerSection()
                    featuresSection()
                    choosePlanSection()
                    actionButton()
                    restorePurchasesButton()
                    privacyPolicyTermsSection()
                    disclaimerTextView()
                    Spacer()
                }
            }
            .animation(.spring(), value: selectedPlan)
        }
    }
    
    private func headerSection() -> some View {
        return HStack(alignment: .top) {
            VStack(alignment: .leading, spacing: 2) {
                Text("Room Scanner Pro")
                    .font(.system(size: 26, weight: .semibold))
                
                Text("Premium")
                    .font(.system(size: 40, weight: .bold))
            }
            Spacer()
            XMarkButton() { dismiss() }
        }
        .padding(20)
    }
    
    private func featuresSection() -> some View {
        return VStack {
            ForEach(0..<features.count) { index in
                HStack {
                    Image(systemName: "checkmark.circle.fill").resizable()
                        .frame(width: 20, height: 20)
                        .foregroundColor(Color.appTheme)
                    Text(LocalizedStringKey(self.features[index]))
                        .font(.system(size: 22, design: .rounded))
                    Spacer()
                }
            }.padding(.leading, 30).padding(.trailing, 30)
        }
    }
    
    private func carouselSection() -> some View {
        return VStack {
            InfiniteScroller(contentWidth: getContentWidth(), direction: .forward) {
                HStack(spacing: 16) {
                    FeatureCardView(featureCard: .unlimitedEntries)
                    FeatureCardView(featureCard: .importData)
                    FeatureCardView(featureCard: .biometricsLock)
                    FeatureCardView(featureCard: .automaticBackups)
                }.padding(8)
            }
            
            InfiniteScroller(contentWidth: getContentWidth(), direction: .backward) {
                HStack(spacing: 16) {
                    FeatureCardView(featureCard: .customTags)
                    FeatureCardView(featureCard: .moreStats)
                    FeatureCardView(featureCard: .moreDates)
                    FeatureCardView(featureCard: .unlimitedGoals)
                }.padding(8)
            }
        }
    }
    
    private func choosePlanSection() -> some View {
        return VStack(alignment: .center, spacing: 16) {
            Text("Choose a plan")
                .foregroundColor(.secondary)
                .font(.system(size: 20, weight: .medium))
                .padding(.top)
            MonthlyPlanView(selectedPlan: $selectedPlan, paymentId: monthsProductId).padding(.bottom, 12)
            YearlyPlanView(selectedPlan: $selectedPlan, paymentId: oneYearProductId).padding(.bottom)
        }
    }
    
    private func actionButton() -> some View {
        return Button {
            if !paymentViewModel.isLoadingPayment{
                
                if let payment = self.paymentViewModel.payments.filter( {$0.paymentId == paymentViewModel.selectedProductId}).first {
                    self.paymentViewModel.startPayment(with: payment.paymentId){ result in
                        if result.success{
                            settingsViewModel.setPremiumUser(paymentId: payment.paymentId)
                            dismiss()
                        }else{
                            alertText = result.error ?? ""
                            showAlert = true
                        }
                    }
                }
                
            }
        } label: {
            if let product = paymentViewModel.payments.filter( {$0.paymentId == paymentViewModel.selectedProductId}).first {
                if product.isTrialAvailable {
                    Text("Start Free Trial")
                        .font(.system(size: 28, weight: .medium))
                        .foregroundColor(.white)
                        .padding(12)
                        .frame(maxWidth: .infinity)
                        .background(Color.appTheme)
                        .cornerRadius(10)
                        .padding(.horizontal, 24)
                }else{
                    Text("Continue")
                        .font(.system(size: 28, weight: .medium))
                        .foregroundColor(.white)
                        .padding(12)
                        .frame(maxWidth: .infinity)
                        .background(Color.appTheme)
                        .cornerRadius(10)
                        .padding(.horizontal, 24)
                }
                
            }
            
        }
        .alert(isPresented: $showAlert){
            Alert(title: Text("Failed"), message: Text(alertText), dismissButton: .default(Text("OK")))
        }
    }
    
    private func restorePurchasesButton() -> some View {
        return Button {
            paymentViewModel.restorePurchases{ result in
                if result.success{
                    settingsViewModel.setPremiumUser(paymentId: "Restore purchase")
                    dismiss()
                }
            }
        } label: {
            Text("Restore Purchases")
                .font(.system(size: 16, weight: .medium))
                .foregroundColor(Color.appTheme)
        }.padding(8)
    }
    
    private func privacyPolicyTermsSection() -> some View {
        return HStack(spacing: 20) {
            Button(action: {
                UIApplication.shared.open(privacyPolicyURL, options: [:], completionHandler: nil)
            }, label: {
                Text("Privacy Policy")
            })
            Button(action: {
                UIApplication.shared.open(termsAndConditionsURL, options: [:], completionHandler: nil)
            }, label: {
                Text("Terms of Use")
            })
        }.font(.system(size: 12)).foregroundColor(.gray).padding()
    }
    
    private func disclaimerTextView() -> some View {
        return VStack {
            Text(AppConfig.DISCLIMER_TEXT).font(.system(size: 12))
                .multilineTextAlignment(.center)
                .padding(.leading, 30).padding(.trailing, 30)
            Spacer(minLength: 50)
        }
    }
    
    private func getContentWidth() -> CGFloat {
        let cardWidth: CGFloat = 90
        let spacing: CGFloat = 16
        let padding: CGFloat = 8
        let contentWidth: CGFloat = (cardWidth * 4) + (spacing * 9) + padding
        return contentWidth
    }
    
    //    private func isIPhone14ProMax() -> Bool {
    //        return screenSize == iPhone14ProMaxScreenSize
    //    }
}

struct PaywallView_Previews: PreviewProvider {
    static var previews: some View {
        PaywallView()
    }
}
