import SwiftUI
import UIKit
import SceneKit

var fileUrl = ""

struct Wall: Identifiable {
    let id = UUID()
    let node: SCNNode
    let measurement: String
    let area: Float
    var isVisible: Bool = true
    var isTextured: Bool = false
    var blocksNeeded: Int = 0
}

struct Floor: Identifiable {
    let id = UUID()
    let node: SCNNode
    var isVisible: Bool = true
    var isTextured: Bool = false
    var blocksNeeded: Int = 0
}

struct RoomDetailView: View {
    var scannedModel: ScannedModel
    @State private var walls: [Wall] = []
    @State private var floor: Floor?
    @State private var selectedWall: Wall?
    @State private var selectedFloor: Floor?
    @State private var totalArea: Float = 0.0
    @State private var roomArea: Float = 0.0 // Yeni değişken oda alanı için
    @State private var floorIsVisible: Bool = true
    @State private var floorIsTextured: Bool = false
    @State private var othersHidden: Bool = false
    @State private var showingImagePicker = false
    @State private var selectedImageName = ""
    @State private var applyingTextureToFloor = false

    var body: some View {
        fileUrl = scannedModel.filePath
        return VStack {
            SceneKitViewContainer(walls: $walls, floor: $floor, selectedWall: $selectedWall, selectedFloor: $selectedFloor, totalArea: $totalArea, roomArea: $roomArea, floorIsVisible: $floorIsVisible, floorIsTextured: $floorIsTextured, othersHidden: $othersHidden).ignoresSafeArea()
            List {
                if let floor = floor {
                    HStack {
                        Text("Floor")
                            .onTapGesture {
                                applyingTextureToFloor = true
                                showingImagePicker = true
                            }
                        Spacer()
                        Text("Blocks needed: \(floor.blocksNeeded)")
                        Spacer()
                        Button(action: {
                            toggleFloorVisibility()
                        }) {
                            Text(floorIsVisible ? "Gizle" : "Göster")
                                .foregroundColor(.blue)
                        }
                    }
                }
                ForEach(walls) { wall in
                    HStack {
                        Text("\(wall.node.name ?? "Unnamed Wall") - \(wall.measurement)")
                            .onTapGesture {
                                selectedWall = wall
                                applyingTextureToFloor = false
                                showingImagePicker = true
                            }
                        Spacer()
                        Text("Blocks needed: \(wall.blocksNeeded)")
                        Spacer()
                        Button(action: {
                            toggleWallVisibility(wall)
                        }) {
                            Text(wall.isVisible ? "Gizle" : "Göster")
                                .foregroundColor(.blue)
                        }
                    }
                }
            }
            Button(action: {
                toggleOthersVisibility()
            }) {
                Text(othersHidden ? "Eşyaları Göster" : "Eşyaları Gizle")
                    .padding()
                    .background(Color.blue)
                    .foregroundColor(.white)
                    .cornerRadius(8)
            }
            Text("Toplam Oda Alanı: \(roomArea, specifier: "%.2f") m²") // roomArea kullanılarak güncellendi
                .padding()
        }
        .sheet(isPresented: $showingImagePicker) {
            ImagePickerView(selectedImageName: $selectedImageName, applyingTextureToFloor: $applyingTextureToFloor) { imageName in
                applyTexture(imageName)
                showingImagePicker = false
            }
        }
    }

    func toggleWallVisibility(_ wall: Wall) {
        if let index = walls.firstIndex(where: { $0.id == wall.id }) {
            walls[index].isVisible.toggle()
            walls[index].node.isHidden = !walls[index].isVisible
        }
    }

    func toggleFloorVisibility() {
        if let floor = floor {
            floorIsVisible.toggle()
            floor.node.isHidden = !floorIsVisible
        }
    }

    func toggleOthersVisibility() {
        othersHidden.toggle()
        NotificationCenter.default.post(name: NSNotification.Name("ToggleOthersVisibility"), object: othersHidden)
    }

    func applyTexture(_ imageName: String) {
        if applyingTextureToFloor, let floor = floor {
            applyTextureToNode(floor.node, with: imageName)
            floorIsTextured.toggle()
        } else if let selectedWall = selectedWall {
            if let index = walls.firstIndex(where: { $0.id == selectedWall.id }) {
                applyTextureToNode(walls[index].node, with: imageName)
                walls[index].isTextured.toggle()
            }
        }
    }

    func applyTextureToNode(_ node: SCNNode, with imageName: String) {
        if let geometry = node.geometry {
            let vertexCount = geometry.sources(for: .vertex).first?.vectorCount ?? 0
            var uvCoordinates: [vector_float2] = []
            for i in 0..<vertexCount {
                let u = Float(i % 2)
                let v = Float((i / 2) % 2)
                uvCoordinates.append(vector_float2(u, v))
            }

            let uvSource = SCNGeometrySource(data: Data(bytes: uvCoordinates, count: uvCoordinates.count * MemoryLayout<vector_float2>.size),
                                             semantic: .texcoord,
                                             vectorCount: vertexCount,
                                             usesFloatComponents: true,
                                             componentsPerVector: 2,
                                             bytesPerComponent: MemoryLayout<Float>.size,
                                             dataOffset: 0,
                                             dataStride: MemoryLayout<vector_float2>.size)

            let elements = geometry.elements.map { element -> SCNGeometryElement in
                return SCNGeometryElement(data: element.data,
                                          primitiveType: element.primitiveType,
                                          primitiveCount: element.primitiveCount,
                                          bytesPerIndex: element.bytesPerIndex)
            }

            var sources = geometry.sources
            sources.append(uvSource)
            let newGeometry = SCNGeometry(sources: sources, elements: elements)

            let material = SCNMaterial()
            if let image = UIImage(named: imageName) {
                material.diffuse.contents = image
                material.diffuse.wrapS = .repeat
                material.diffuse.wrapT = .repeat
            } else {
                print("Image not found: \(imageName)")
            }
            material.isDoubleSided = true
            material.diffuse.mipFilter = .linear
            material.diffuse.minificationFilter = .linear
            material.diffuse.magnificationFilter = .linear
            newGeometry.materials = [material]

            node.geometry = newGeometry

            // Gerekli blok sayısını hesapla ve güncelle
            let area = calculateArea(of: node)
            let blocksNeeded = Int(ceil(area))
            if applyingTextureToFloor {
                floor?.blocksNeeded = blocksNeeded
            } else if let selectedWall = selectedWall, let index = walls.firstIndex(where: { $0.id == selectedWall.id }) {
                walls[index].blocksNeeded = blocksNeeded
            }
        }
    }

    func calculateArea(of node: SCNNode) -> Float {
        if let geometry = node.geometry {
            let boundingBox = geometry.boundingBox
            let min = boundingBox.min
            let max = boundingBox.max
            let width = (max.x - min.x)
            let length = (max.z - min.z) // Yükseklik yerine derinliği kullan
            return width * length
        }
        return 0
    }
}

struct ImagePickerView: View {
    @Binding var selectedImageName: String
    @Binding var applyingTextureToFloor: Bool
    let images = ["marble.jpg", "marble1.jpg", "stone.jpg", "flr.jpg", "floor.jpg"]
    let onSelect: (String) -> Void

    var body: some View {
        NavigationView {
            VStack {
                List(images, id: \.self) { imageName in
                    HStack {
                        if let uiImage = UIImage(named: imageName) {
                            Image(uiImage: uiImage)
                                .resizable()
                                .scaledToFit()
                                .frame(width: 50, height: 50)
                        } else {
                            Rectangle()
                                .fill(Color.gray)
                                .frame(width: 50, height: 50)
                        }
                        Text(imageName)
                    }
                    .onTapGesture {
                        selectedImageName = imageName
                        onSelect(imageName)
                    }
                }
                .navigationBarTitle("Select Texture", displayMode: .inline)
                .navigationBarItems(trailing: Button("Done") {
                    onSelect(selectedImageName)
                })
            }
        }
    }
}

struct SceneKitViewContainer: UIViewRepresentable {
    @Binding var walls: [Wall]
    @Binding var floor: Floor?
    @Binding var selectedWall: Wall?
    @Binding var selectedFloor: Floor?
    @Binding var totalArea: Float
    @Binding var roomArea: Float // Yeni binding oda alanı için
    @Binding var floorIsVisible: Bool
    @Binding var floorIsTextured: Bool
    @Binding var othersHidden: Bool

    func printNodeHierarchy(node: SCNNode, level: Int = 0) {
        let indent = String(repeating: "  ", count: level)
        print("\(indent)\(node.name ?? "Unnamed Node") - Geometry: \(node.geometry != nil ? "Yes" : "No")")
        for child in node.childNodes {
            printNodeHierarchy(node: child, level: level + 1)
        }
    }

    func findWallsAndFloor(in node: SCNNode) -> ([Wall], Floor?) {
        var foundWalls: [Wall] = []
        var foundFloor: Floor? = nil
        if let geometry = node.geometry {
            if node.name?.contains("Wall") == true {
                let (measurement, area) = calculateMeasurement(for: geometry)
                foundWalls.append(Wall(node: node, measurement: measurement, area: area))
            } else if node.name?.contains("Floor") == true {
                foundFloor = Floor(node: node)
            } else {
                node.name = "others" // Diğer nesneleri etiketle
                node.geometry?.firstMaterial?.diffuse.contents = nil // Renksiz yap
            }
        }
        for child in node.childNodes {
            let (walls, floor) = findWallsAndFloor(in: child)
            foundWalls.append(contentsOf: walls)
            if foundFloor == nil {
                foundFloor = floor
            }
        }
        return (foundWalls, foundFloor)
    }

    func calculateMeasurement(for geometry: SCNGeometry) -> (String, Float) {
        let boundingBox = geometry.boundingBox
        let min = boundingBox.min
        let max = boundingBox.max
        let width = (max.x - min.x)
        let height = (max.y - min.y)
        let area = width * height
        let measurement = String(format: "Width: %.2f meters, Height: %.2f meters", width, height)
        return (measurement, area)
    }

    func makeUIView(context: Context) -> SCNView {
        let scnView = SCNView(frame: .zero)
        scnView.allowsCameraControl = true
        scnView.autoenablesDefaultLighting = true
        scnView.backgroundColor = UIColor.gray

        guard let sceneURL = URL(string: fileUrl) else {
            print("Invalid URL")
            return scnView
        }
        guard let scene = try? SCNScene(url: sceneURL) else {
            print("Failed to load scene")
            return scnView
        }
        scnView.scene = scene

        printNodeHierarchy(node: scene.rootNode)
        
        DispatchQueue.main.async {
            let (walls, floor) = findWallsAndFloor(in: scene.rootNode)
            self.walls = walls
            self.floor = floor
            self.roomArea = calculateTotalRoomArea(floor: floor) // Oda alanını hesapla ve ayarla
        }

        NotificationCenter.default.addObserver(forName: NSNotification.Name("ToggleOthersVisibility"), object: nil, queue: .main) { notification in
            if let hide = notification.object as? Bool {
                self.toggleOthersVisibility(in: scene.rootNode, hide: hide)
            }
        }

        return scnView
    }

    func updateUIView(_ uiView: SCNView, context: Context) { }

    func makeCoordinator() -> Coordinator {
        Coordinator(self)
    }

    class Coordinator: NSObject {
        var parent: SceneKitViewContainer

        init(_ parent: SceneKitViewContainer) {
            self.parent = parent
        }
    }

    func toggleOthersVisibility(in node: SCNNode, hide: Bool) {
        for child in node.childNodes {
            if child.name == "others" {
                child.isHidden = hide
            }
            toggleOthersVisibility(in: child, hide: hide)
        }
    }

    func calculateTotalRoomArea(floor: Floor?) -> Float {
        guard let floor = floor else {
            return 0.0
        }
        return calculateFloorArea(of: floor.node)
    }
    
    func calculateFloorArea(of node: SCNNode) -> Float {
        guard let geometry = node.geometry else {
            print("Geometry yok")
            return 0.0
        }
        
        let vertexSources = geometry.sources(for: .vertex)
        guard let vertexSource = vertexSources.first else {
            print("Vertex kaynakları yok")
            return 0.0
        }
        
        let vertexCount = vertexSource.vectorCount
        let vertices = vertexSource.data.withUnsafeBytes { (pointer: UnsafePointer<vector_float3>) -> [vector_float3] in
            return Array(UnsafeBufferPointer(start: pointer, count: vertexCount))
        }
        
        print("Vertex sayısı: \(vertices.count)")
        for vertex in vertices {
            print("Vertex: \(vertex)")
        }
        
        if vertices.count % 3 == 0 {
            return calculateTriangleMeshArea(vertices: vertices)
        } else {
            print("Vertex sayısı üçgen değil")
            return 0.0
        }
    }

    func calculateTriangleMeshArea(vertices: [vector_float3]) -> Float {
        var totalArea: Float = 0.0
        for i in stride(from: 0, to: vertices.count, by: 3) {
            let v0 = vertices[i]
            let v1 = vertices[i + 1]
            let v2 = vertices[i + 2]
            
            // Üçgenin doğrusal olup olmadığını kontrol et
            if isCollinear(v0, v1, v2) {
                print("Üçgen doğrusal, atlanıyor: \(v0), \(v1), \(v2)")
                continue
            }
            
            totalArea += calculateTriangleArea(v0: v0, v1: v1, v2: v2)
        }
        print("Toplam Üçgen Alanı: \(totalArea)")
        return totalArea
    }
    
    func isCollinear(_ v0: vector_float3, _ v1: vector_float3, _ v2: vector_float3) -> Bool {
        let tolerance: Float = 1e-5
        let area = abs((v0.x * (v1.y - v2.y)) + (v1.x * (v2.y - v0.y)) + (v2.x * (v0.y - v1.y)))
        return area < tolerance
    }

    func calculateTriangleArea(v0: vector_float3, v1: vector_float3, v2: vector_float3) -> Float {
        let a = distance(v0, v1)
        let b = distance(v1, v2)
        let c = distance(v2, v0)
        let s = (a + b + c) / 2.0
        let area = sqrt(s * (s - a) * (s - b) * (s - c))
        print("Üçgen Alanı: \(area)")
        return area
    }
    
    func distance(_ v1: vector_float3, _ v2: vector_float3) -> Float {
        return sqrt(pow(v1.x - v2.x, 2) + pow(v1.y - v2.y, 2) + pow(v1.z - v2.z, 2))
    }
}
